__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__",
]

__title__ = "mysmbus"
__summary__ = 'This Python module allows SMBus access through the I2C /dev interface on Linux hosts. The host kernel must have I2C support, I2C device interface support, and a bus adapter driver.'
__uri__ = 'https://github.com/bivab/smbus-cffi'

__version__ = "0.1"

__author__ = "David Schneider"
__email__ = "david.schneider@bivab.de"

__license__ = 'GPLv2'
